%% 1. Variable Names
% It is sensible to use meaningful variable names wherever possible. This
% helps other people understand what your code is doing, and therefore
% check for any bugs or mistakes. To take a simple example, compare:

x = 5;
y = 4;
z = x * y;

% ...with:

height = 5;
width = 4;
area = height * width;

% The latter example if much easier to interpret, and therefore check for
% accuracy. This also means that, if you come back to the code after a long
% break, you will be quicker to understand what the code is doing, and why.

% You can also use annotation when assigning variables to remind yourself
% what the variable reprsents, or why it is being created at all. For
% example:

height = 5;   % Height of the rectangle (cm)
width = 4;   % Width of the rectangle (cm)
area = height * width;   % Compute the area of the rectangle (cm^2)

% However, meaningful naming (like anything in life) should not become a 
% dogma - there are limits! Short variable names are always preferable, and
% so:

Computed_Area_of_Arbitrary_Rectangle = height * width;

% ...is NOT a good choice. This variable name is unwieldly, hard to parse, 
% and  takes up too much space in the script and in the workspace. 

% Remember, you are free to use your own abbreviations and concatenations, 
% so you can come up with your own rules, as long as you will remember what
% each variable name relates to. For example, compare:

subjects_means_variance = rand(10,2);

% ...with:

SubsAvgVar = rand(10,2);

% It is useful to try and be consistent within your own scripts, however.
% For example, if you always use 'S' for subject, then you will know what
% that means when you come back to an old script, or when you have to
% explain a script to someone else. Again, annotation always helps. 



%% 2. Alignment
% In addition to obvious or meaningful variable names, alignment can help
% to make a script easier to read and interpret. You can use the 'tab' key
% to move through a series of aligned locations on the screen, which is 
% useful for aligning equality signs or annotations in consecutive lines of 
% code. For example, compare:

heights = rand(5,1)*2;   % Height of the shapes (0-2 cm)
widths = rand(5,1)*4;   % Width of the shapes (0-4 cm)
depths = rand(5,1)*10;   % Depth of the shapes (0-10 cm)
areas = heights.*widths;   % Compute the face area of the shapes (cm^2)
volumes = areas.*depths;   % Compute the volume of the shapes (cm^3)

% ...with:

heights = rand(5,1) * 2;        % Height of the shapes (0-2 cm)
widths  = rand(5,1) * 4;        % Width of the shapes (0-4 cm)
depths  = rand(5,1) * 10;       % Depth of the shapes (0-10 cm)
areas   = heights .* widths;    % Compute the face area of the shapes (cm^2)
volumes = areas .* depths;      % Compute the volume of the shapes (cm^3)

% The code is identical, but the latter example is easier to read and
% interpret. Again, this should not become a dogma - use this convention 
% within reason. If one or two out of many assignments take up more space 
% on the line, then break the rule. For example, compare:

heights                                                         = rand(5,1)*2;          % Height of the shapes (0-2 cm)
widths                                                          = rand(5,1)*4;          % Width of the shapes (0-4 cm)
depths                                                          = rand(5,1)*10;         % Depth of the shapes (0-10 cm)
areas                                                           = heights .* widths;    % Compute the face area of the shapes (cm^2)
VolumesOfRandomShapesWithAnExaggeratedFilenameToProveThePoint   = areas .* depths;      % Compute the volume of the shapes (cm^3)

% ...with:

heights = rand(5,1)*2;          % Height of the shapes (0-2 cm)
widths  = rand(5,1)*4;          % Width of the shapes (0-4 cm)
depths  = rand(5,1)*10;         % Depth of the shapes (0-10 cm)
areas   = heights .* widths;    % Compute the face area of the shapes (cm^2)
VolumesOfRandomShapesWithAnExaggeratedFilenameToProveThePoint = areas .* depths;        % Compute the volume of the shapes (cm^3)



%% 3. Smart Indentation
% While Matlab will not align your code for you, it does perform 'smart
% indentation', which means that loops and conditional statements (i.e. 
% 'if', 'switch / case', 'for', and 'while' statements) will be organised 
% such that the start and end of each loop have the same indentation. You 
% will probably have noticed this in the previous lecture, or if you have
% written a loop or conditional argument in this Editor window. For 
% example, try copying the text below underneath it in this script:

for shape = 1 : size(heights,1)
    if heights(shape) >= 1
        disp(['Shape number ' int2str(shape) ' is taller than 1cm']);
    end    
end

% Hopefully, you will have noticed that - as you type the code out - each 
% new line starts one 'tab' further in after you enter the 'for' and 'if' 
% statements, and each 'end' statement brings the line one 'tab' further 
% backward towards the margin, so that the start and end of each loop or
% conditional argument is aligned on the screen. 

% Note that it is possible to enter and run the code without this 'smart
% indentation', without making any difference to the output. However, the 
% code is much easier to interpret when smart indentation is used. 

% If you have a section of code which is not indented correctly (perhaps 
% because you have inserted an additional loop within code that was already
% correctly indented), then you can apply smart indentation by highlighting
% all of the code and pressing ctrl+I



%% 4. Organising Code
% There are several simple guidelines for organising your code when you are
% writing scripts that will help you to keep track of what you are doing,
% as well as make your code easier to interpret, share and re-visit.

% Some of these rules will seem needlessly complicated in the simple 
% examples we consider here, but when you come to writing more complex 
% scripts (like the psychologial experiments you will program in future 
% lectures), you will find these simple rules to be invaluable.

% For that reason, we're going to need a more complex code example, and so 
% we will create a sample psychology experiment script that automatically 
% generates data, rather than collecting it from real participants. In this
% example, we will randomly generate reaction times and responses (i.e. key
% presses) for ten participants, in two conditions that each contain twenty
% trials, presented in a random order. We will then calculate the mean
% reaction time and most common key press for each participant.

% A good analogy for writing good code is writing a clear recipe. A clear 
% recipe has the following structure:
% (1) A brief description of the 'output' - i.e. the name of the dish, how 
% many people it serves, and potentially the cost per person or nutritional
% information. 
% (2) The name and quantity of all the ingredients and equipment that is 
% needed.
% (3) An outline of how those ingredients are combined to generate the 
% final product.

% So when we write code, we should follow a similar structure. Now let's
% stretch that analogy to breaking point...



%% 5. Start with high-level description / help information
% First, have a high-level description at the start of the code - what does
% the piece of code do, what inputs does it require, and what are the
% outputs that it produces? You will have seen examples of this in the
% functions we wrote in the previous lecture.

% It is also often a good idea to include details of when you wrote the 
% code, what changes you made from previous versions, and your contact 
% details in this header information. This will be useful if you share the
% code with other people, or if you generate several versions of a similar
% piece of code that do slightly different things, and then re-visit that
% code at a later date

% As mentioned in the previous lecture, this text is also what will appear
% if anyone calls up the 'help' information for your code. For example,
% make sure this script is in your current folder, and then type:

help AdvancedProgramming

% Hopefully, you should see the first section of annotation from the top of
% this script.

% For another example of this, have a look at any built-in Matlab function,
%  i.e. have a look at the built-in Matlab script for computing the mean:

edit mean

% First, observe the header information that is provided at the top of that
% script. Then, see what happens if you type:

help mean

% Similarly, for our sample psychology experiment code, we should include 
% something along the lines of:

% Sample psychology experiment script, Daniel Bush (UCL, 2015)
% This script randomly generates reaction times and key press responses for
% ten 'participants' in two 'conditions' that each contain twenty 'trials'
% presented in a random order. The purpose of this script is to demonstrate
% good coding practice. Contact: d.bush AT ucl.ac.uk



%% 6. Then define constant variables and assign memory
% Second, define the input variables that the code requires and what these 
% inputs correspond to, using annotation. 

% In particular, if there are certain constants that you will use 
% repeatedly throughout the code, then it is always a good idea to assign 
% those constants to a variable at the very start of the code. If you do 
% that, then should you ever need to change the value of that constant, you 
% can easily edit the line where you define that variable, at the top of 
% your code, rather than having to scan through the code and change all 
% instances of that constant. 

% In our example, there are several constants we will use multiple times in
% our script - namely, the number of participants, conditions and trials 
% that we will need to loop through when we generate our data. Hence, we 
% would continue our script with something along the lines of:

n_participants  = 10;   % Number of simulated participants
n_conditions    = 2;    % Number of simulated conditions
n_trials        = 20;   % Number of simulated trials per condition

% If we suddenly decide we want to change the number of trials or
% conditions, then we just need to change the value of these variables 
% once, and the code will work perfectly, rather than having to scan 
% through the whole script and change each '20' or '2' etc.

% It is also good practice to assign memory at the start of a script for
% any variables that are going to be entered in a loop within the code 
% itself - a bit like specifying the size of the baking tin you need for 
% the cake you are preparing in a recipe (or maybe I'm stretching the 
% analogy a bit thin now...). 

% Practically speaking, this means creating 'empty' variables of the size 
% you will need, which values generated by the code can then be entered 
% into later on. This allows the computer to optimally assign memory for 
% storing each of those variables, and makes your code run faster. 

% In our example, we want to collect reaction times and key presses for 10
% participants across 20 x 2 = 40 trials. We would also like to record the
% random order for each participant in which trials from each condition are
% presented. Hence, we need to create three variables for collecting this 
% information. Each of these empty variables will have 10 rows (one for
% each participant), and 40 columns (one for each trial):

RTs             = zeros(n_participants,n_conditions*n_trials);  % Assign memory for collecting reaction times
KeyPresses      = zeros(n_participants,n_conditions*n_trials);  % Assign memory for collecting key presses
TrialConds      = zeros(n_participants,n_conditions*n_trials);  % Assign memory for recording which condition each trial belongs to

% Note that we have used the Matlab function 'zeros' to create these
% 'empty' variables. The function 'zeros' simply creates a vector or matrix
% of a specified size in which every entry is a zero. Just like when 
% indexing a matrix, the number of rows is defined first (i.e. is the first
% input) and the number of columns is defined second (i.e. is the second 
% input). At the start of the script, each entry in this matrix is a zero, 
% but we can replace these zeros with the randomly generated values as we
% go through the loops of the main script.

% There are a couple of other Matlab functions which will be useful when
% defining variables and assigning memory. Firstly, instead of zeros, we
% can also assign memory by generating matrices full of 'NaN' values. 'NaN'
% stands for 'not a number', and can be useful to highlight errors in the
% code. For example, try running the following lines of code:

test = [2 5 8 NaN];
mean(test)

% You'll see that the mean is equal to 'NaN'. This is because there is a 
% 'NaN' in the matrix. If there is a single 'NaN' entry in any vector or 
% matrix, then any Matlab function that is performed on that vector or
% matrix will produce the output 'NaN' (i.e. also try computing the minimum
% and maximum values, or the standard deviation of the vector). 

% This can be useful, however, because we know that - in this example - 
% every trial should generate a reaction time. Now, let's suppose that the
% empty matrix we generate at the start of the code is all zeros. If 
% something goes wrong - such as our fictional participant not responding 
% in a certain trial, then no entry will be made in the 'RTs' matrix for 
% that participant and trial. There is no reason we would notice this when 
% we run the code (it will not produce an error), but when we compute the 
% mean reaction time for each participant at the end, the true mean will be
% underestimated for that participant, as one trial will have a reaction
% time of zero. Conversely, if we start with RTs as an empty matrix of 
% 'NaN' values, then the mean RT will be 'NaN' for that participant. This
% will alert us to the fact that something has gone wrong, and we can go
% back and compute the real mean. We do this by using the function
% 'nanmean', which computes the mean only for entries where the value is
% not a 'NaN':

nanmean(test)

% You should see that this computes the actual mean of the three non-NaN
% values in the vector 'test'. Hence, if we start with a matrix of NaNs 
% for our RTs, then we can compute the correct mean reaction time for each
% participant at the end of the script with 'nanmean', even if our
% 'participants' do not respond in some trials. 

% Another useful function when dealing with vectors or matrices that
% contain 'NaN' values is 'isnan' - this simply tells you whether or not
% each entry in the vector or matrix is a NaN or not. In this example, we
% could use it to count how many of the trials our participant's responded
% to. In this case, we'd want to know how many entries in the matrix are 
% NOT NaNs. Hence, we can use '~', which signifies the logical operation 
% 'NOT'. For example:

sum(~isnan(test))

% ...asks Matlab to add up (i.e. 'sum') the number of entries in the vector
% 'test' that are NOT NaNs. This works because the code:

~isnan(test)

% ...generates a logical array (i.e. with all entries either equal to '1' 
% where the expression is true and equal to '0' where the expression is 
% false). Summing that array (i.e. adding up all the values within it)
% therefore tells us the number of entries that were true (i.e. equal to
% '1'). In the example below, we will use the number of non-NaN entries in
% 'RTs', along with the total number of trials, to compute the percentage 
% of trials that each participant responded to. 

% However, it is also important to note that not all Matlab functions have
% a NaN equivalent (in the sense that 'nanmean' is to 'mean'). As we will
% see in the example below, there is no 'nanmode' function that we can use
% to get the mode (i.e. most frequent) key press for each participant.

% Finally, one other function that is useful for assigning values to
% matrices is 'ones'. Like 'zeros', 'ones' creates an arbitrary sized
% matrix where every entry is a 1. We can multiply a matrix of ones that we
% create with this function by any number to get a matrix where every value
% is equal to that number, and we can combine that with the concatenation
% (i.e. 'stacking') of matrices in square brackets in order to create
% matrices that consist of smaller matrices of different numbers. We will
% see below how this is useful for assigning condition labels to each
% trial, but for now, let's try a few examples:

AllOnes         = ones(10,10);
AllTwos         = 2*ones(10,10);
TheOnesAndTwos  = [ones(10,5) 2*ones(10,5)];

% Have a look at each of these variables in the Variable Editor, and play
% around with the function yourself to get an idea of how it works. Don't
% forget, the built-in Matlab help files are always a useful resource when
% learning about new functions too. For now, however, back to the main
% focus of the lecture...



%% 7. Then write a high-level annotated outline of the rest of your code
% Third, on the subject of annotation, when you have to write a large piece
% of code, the best way to start is to write a 'high-level' outline of the 
% whole script in annotation, so that you can get the order of each section
% clear in your head and work out which constant variables and memory
% variables you will need. Once you have that annotated 'plan', you can
% then fill in the 'low-level' details. 

% In our psychology experiment example, for instance, we generate an
% outline such as:


% Assign constant variables for the experiment - this will include the 
% number of participants, conditions, trials

% Assign memory for the output of the experiment - this will include 
% reaction times, key presses, and the condition for each trial for each
% participant

% Loop through each participant, randomly assign a condition to each trial,
% and store the condition for each trial for that participant

% Loop through each trial, randomly generate reaction times and key 
% presses for that trial and store that information too

% Compute the mean reaction time and most common key press for each 
% participant 

% Clear variables that are not needed as output


% Once we have that outline, we can start to fill in the actual code and
% finer details, with a clearer idea in our head of the structure of the
% entire script:


% Sample psychology experiment script, Daniel Bush (UCL, 2015)
% This script randomly generates reaction times and key press responses for
% ten 'participants' in two 'conditions' that each contain twenty 'trials'
% presented in a random order. The purpose of this script is to demonstrate
% good coding practice. Contact: d.bush AT ucl.ac.uk

% Assign constant variables for the experiment
n_participants  = 10;   % Number of simulated participants
n_conditions    = 2;    % Number of simulated conditions
n_trials        = 20;   % Number of simulated trials per condition

% Assign memory for the output of the experiment
RTs             = nan(n_participants,n_conditions*n_trials);        % Assign memory for collecting reaction times
KeyPresses      = nan(n_participants,n_conditions*n_trials);        % Assign memory for collecting key presses
TrialConds      = zeros(n_participants,n_conditions*n_trials);      % Assign memory for recording which condition each trial belongs to

% Loop through each participant
for p = 1 : n_participants
        
    trial_cond      = [ones(1,n_trials) 2*ones(1,n_trials)];        % Assign a condition to each trial
    trial_cond      = trial_cond(randperm(n_conditions*n_trials));  % Randomise the order of trials
    TrialConds(p,:) = trial_cond;                                   % Store the condition for each trial for that participant
    clear trial_cond
    
    % Loop through each trial
    for t = 1 : n_trials*n_conditions
        
        if rand <= 0.95                         % Simulated participants respond on each trial with a 95% probability            
            RTs(p,t)        = 400 + rand*500;   % Generate a random reaction time between 400 and 900ms
            KeyPresses(p,t) = round(rand*5);    % Generate a random key press between 0 and 5
        end
    end
    clear t
    
end
clear p

Responded       = sum(~isnan(RTs),2);           % Compute the number of trials on which each participant responded
MeanRTs         = nanmean(RTs,2);               % Compute the mean reaction time for each participant
ModeKeyPresses  = mode(KeyPresses,2);           % Compute the most frequency key press for each participant
clear n_participants n_conditions n_trials      % Clear variables that are not needed as output


% It is also important to note again, at this stage, that we are giving our
% variables sensible names that aren't too long or unwieldly; aligning the
% equality signs and annotations on adjacent lines to make them easier to
% read, and annotating each variable to remind ourselves why it is being
% created. 



%% 8. Use white space and code 'cells' to organise your scripts
% Fourth, as demonstrated in the sample psychological experiment code
% above, and in these sample scripts from each lecture, it is a good idea 
% to use blank lines or code 'cells' (which begin with '%%', as introduced 
% in the previous lecture) to separate out the different sections of your 
% script - such as defining constant variables, assigning memory etc. 

% This makes the code easier to read, and also gives you another chance to 
% add some annotation, by giving each section or cell a brief 'heading' 
% that provides information about what that part of the code does. 

% Finally, as demonstrated in the example above, another good tip is to 
% clear any variables that are no longer needed. This both helps to save 
% memory, so that your code will run faster, and keep your workspace 
% 'tidy', so that there are not lots of unused variables to wade through if
% you are looking for a particular output of your code. 

% The most frequent example of this is clearing the 'counting' variables 
% that you use in loops (such as 'p' for the participant loop and 't' for 
% the trial loop above), as well as 'temporary' variables that you create 
% in the process of computing something else (such as 'trial_cond' above, 
% which is used to randomise the condition for each trial). 

% Try and get in the habit of immediately clearing the counting variable
% every time you 'end' a loop you have written, and looking through the
% variables you have created in each section of code and clearing those
% which will not be used in the remainder of your script.



%% 9. Always test your code with simple worked examples!
% Once you have written your code, it is always a good idea to test it a
% few times with worked examples. Don't forget that code can not do what
% it's supposed to, and still not generate an error. 

% One easy way to run through some worked examples and check that your code
% is working properly, is to take advantage of Matlab's debugging mode.
% Simply enter a breakpoint at the first line of code, then run the script,
% and proceed through the code one step at a time, using the workspace or
% plotting functions (which you will be introduced to in future lectures!)
% to make sure that the variables being created, or operations being
% performed on those variables, are correct.

% This approach also means that - if there are any mistakes - you will
% immediately find out where they are, rather than noticing that your code
% is not producing the end product it is supposed to, without knowing which
% part of the script has gone wrong.



%% 10. Finally, while we're here, and have some simulated data, let's do a 
% quick recap of how to index, manipulate, and analyse this data. The most
% obvious thing we may want to do with this data is compute the mean
% reaction time across participants for each condition. To do this, we
% need to use indices computed from the 'TrialConds' array to access
% specific entries in the 'RTs' array. There are lots of different ways to
% do this, with varying levels of simplicity / elegance. Let's start with
% the most straightforward and intuitive (but also the slowest and most
% long-winded), using the 'find' function:

condRTs     = nan(size(TrialConds,1),2);            % Assign some memory for the mean reaction times by condition
for p       = 1 : size(TrialConds,1)                % For each row of 'TrialConds' (i.e. for data from each participant)
    cond1trials     = find(TrialConds(p,:)==1);     % Find all the columns in that row of 'TrialConds' that are equal to one (i.e. all the trials from condition one)
    cond2trials     = find(TrialConds(p,:)==2);     % Do the same for condition two
    condRTs(p,1)    = nanmean(RTs(p,cond1trials));  % Work out that participant's mean RT for all condition 1 trials
    condRTs(p,2)    = nanmean(RTs(p,cond2trials));  % Work out that participant's mean RT for all condition 2 trials
    clear cond1trials cond2trials                   % Clear the variables we no longer need
end
clear p

% As you can see, we loop through the data for each participant and use the
% 'find' function to find all of the trials for that participant that fall
% within each condition, by looking at the relevant row of 'TrialConds'. 
% Once we have extracted that list of column numbers, we can use it to 
% access the corresponding entries in the same row of the 'RTs' variable,
% and take the mean (using the nanmean function to avoid any issues that
% arise from trials where participants did not respond, as noted above). 

% This method works perfectly well, but as you can see, even Matlab knows
% there is a better way to do this. The 'cond1trials' and 'cond2trials' 
% variable names should be highlighted orange and underlined in the code 
% above (on lines 501 and 502), showing that Matlab is giving a warning. 
% That warning tells you that logical indexing can be used to improve
% performance (and requires fewer lines of code). To show how this is
% achieved, we'll first edit the lines of code that 'find' which trials
% fall within each condition:

condRTs     = nan(size(TrialConds,1),2);            % Assign some memory for the mean reaction times by condition
for p       = 1 : size(TrialConds,1)                % For each row of 'TrialConds' (i.e. for data from each participant)
    cond1trials     = TrialConds(p,:)==1;           % Find all the columns in that row of 'TrialConds' that are equal to one (i.e. all the trials from condition one)
    cond2trials     = TrialConds(p,:)==2;           % Do the same for condition two
    condRTs(p,1)    = nanmean(RTs(p,cond1trials));  % Work out that participant's mean RT for all condition 1 trials
    condRTs(p,2)    = nanmean(RTs(p,cond2trials));  % Work out that participant's mean RT for all condition 2 trials
    clear cond1trials cond2trials                   % Clear the variables we no longer need
end
clear p

% As you can see, we no longer use the 'find' function, but simply use the
% expression 'TrialConds(p,:)==1' (or 2) to generate a logical array with
% values of '1' (i.e. 'true') for every entry in that row of 'TrialConds'
% that has a value of 1 or 2 - i.e. to be true for all trials for that
% participant where the condition was equal to 1 or 2. We can then use that
% logical array to access those same indices in the 'RTs' variable, and
% work out the mean RT for each condition for that participant. This code
% will run a little faster, and is a little neater. In fact, we can now
% combine the logical indexing with the 'nanmean' lines, and hence avoid
% the need for the 'cond1trials' and 'cond2trials' variables:

condRTs     = nan(size(TrialConds,1),2);                    % Assign some memory for the mean reaction times by condition
for p       = 1 : size(TrialConds,1)                        % For each row of 'TrialConds' (i.e. for data from each participant)
    condRTs(p,1)    = nanmean(RTs(p,TrialConds(p,:)==1));   % Work out that participant's mean RT for all condition 1 trials
    condRTs(p,2)    = nanmean(RTs(p,TrialConds(p,:)==2));   % Work out that participant's mean RT for all condition 2 trials    
end
clear p

% This is neater, will run even quicker, and uses even less memory.
% Although the lines of code within the 'for' loop seem complex, hopefully
% the examples above help you to understand how they were constructed. 

% We could use the same syntax to extract other statistics from the
% simulated data - for example, count the number of key presses that are
% equal to 0 and 5 in each condition. Again, we'll compact the code across
% two examples, to hopefully give you some intuition about how the more
% complex / more compact version arises from the more straightforward /
% more long-winded version. First, the longer version:

condRTs     = nan(size(TrialConds,1),2);                        % Assign some memory for the mean reaction times by condition
zeroPresses = nan(size(TrialConds,1),2);                        % Assign some memory for the number of '0' key presses by condition
fivePresses = nan(size(TrialConds,1),2);                        % Assign some memory for the number of '5' key presses by condition
for p       = 1 : size(TrialConds,1)                            % For each row of 'TrialConds' (i.e. for data from each participant)
    cond1trials         = TrialConds(p,:)==1;                   % Find all the columns in that row of 'TrialConds' that are equal to one (i.e. all the trials from condition one)
    cond2trials         = TrialConds(p,:)==2;                   % Do the same for condition two

    condRTs(p,1)        = nanmean(RTs(p,cond1trials));          % Work out that participant's mean RT for all condition 1 trials
    condRTs(p,2)        = nanmean(RTs(p,cond2trials));          % Work out that participant's mean RT for all condition 2 trials    

    zeroPresses(p,1)    = sum(KeyPresses(p,cond1trials)==0);    % Work out the number of '0' key presses for all condition 1 trials
    zeroPresses(p,2)    = sum(KeyPresses(p,cond2trials)==0);    % Work out the number of '0' key presses for all condition 2 trials
    
    fivePresses(p,1)    = sum(KeyPresses(p,cond1trials)==5);    % Work out the number of '5' key presses for all condition 1 trials
    fivePresses(p,2)    = sum(KeyPresses(p,cond2trials)==5);    % Work out the number of '5' key presses for all condition 2 trials
    clear cond1trials cond2trials                               % Clear the variables we no longer need
end
clear p

% Then, the shorter version, which does away with the need for the
% 'cond1trials' and 'cond2trials' variables:

condRTs     = nan(size(TrialConds,1),2);                            % Assign some memory for the mean reaction times by condition
zeroPresses = nan(size(TrialConds,1),2);                            % Assign some memory for the number of '0' key presses by condition
fivePresses = nan(size(TrialConds,1),2);                            % Assign some memory for the number of '5' key presses by condition
for p       = 1 : size(TrialConds,1)                                % For each row of 'TrialConds' (i.e. for data from each participant)    
    condRTs(p,1)        = nanmean(RTs(p,TrialConds(p,:)==1));       % Work out that participant's mean RT for all condition 1 trials
    condRTs(p,2)        = nanmean(RTs(p,TrialConds(p,:)==2));       % Work out that participant's mean RT for all condition 2 trials    

    zeroPresses(p,1)    = sum(KeyPresses(p,TrialConds(p,:)==1)==0); % Work out the number of '0' key presses for all condition 1 trials
    zeroPresses(p,2)    = sum(KeyPresses(p,TrialConds(p,:)==2)==0); % Work out the number of '0' key presses for all condition 2 trials
    
    fivePresses(p,1)    = sum(KeyPresses(p,TrialConds(p,:)==1)==5); % Work out the number of '5' key presses for all condition 1 trials
    fivePresses(p,2)    = sum(KeyPresses(p,TrialConds(p,:)==2)==5); % Work out the number of '5' key presses for all condition 2 trials    
end
clear p

% Remember, in all cases, we are just using the normal brackets, after a
% variable name, to access specific entries within that array. The first
% number after the brackets gives the row number(s), the second gives the
% column number(s). However, in the case of logical indexing, the column
% numbers are not a list of numbers, but an array of the same size, with
% values of '1' (or true) for columns we wish to access, and values of '0'
% (or false) for those we wish to ignore. 